unit udmListener;

interface

uses
   windows            // Delphi VCL
  ,SysUtils           // Delphi VCL
  ,stdctrls           // Delphi VCL
  ,controls           // Delphi VCL
  ,forms              // Delphi VCL
  ,Classes            // Delphi VCL
  ,IdAntiFreezeBase   // Part of Indy Delphi Components Suite
  ,IdAntiFreeze       // Part of Indy Delphi Components Suite
  ,IdBaseComponent    // Part of Indy Delphi Components Suite
  ,IdComponent        // Delphi VCL
  ,JanXMLParser2      // Part of EasyXML 2.1
  ,IdStackConsts      // Part of Indy Delphi Components Suite
  ,IdIOHandlerSocket  // Part of Indy Delphi Components Suite
  ,IdTCPServer        // Part of Indy Delphi Components Suite
  ;

type
  tempconfigobject = Class
    subson    : Boolean;
    Framerate : Real;
    Shutdown  : Boolean;
  public
    constructor create;
  end;

  TdmListener = class(TDataModule)
    IdTCPServer1: TIdTCPServer;
    IdAntiFreeze1: TIdAntiFreeze;
    procedure IdTCPServer1Disconnect(AThread: TIdPeerThread);
    procedure IdTCPServer1Connect(AThread: TIdPeerThread);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    Fshares: TJanXMLNode2;
    downloadiplist : TStringlist;
    IpCurrentPath : TStringlist;
    ipConfigList : TStringlist;
    Function DoConfig(aUserIP:String;aConfigOption:String):TempConfigObject;
  public
    property shares : TJanXMLNode2 read Fshares write FShares;
    Procedure Log(debuglevel:Integer;S:STring;AThread: TIdPeerThread;aIP:String = '000.000.000.000');
  end;

implementation

uses
  main            // RelaX Main unit
 ,AviInfo         // RelaX avi info
 ,downloadframe   // RelaX download information frame
 ,ump3object      // RelaX mp3 info
 ,uGeneral        // RelaX general functions / procedures
 ,dialogs         // Delphi vcl
 ,uSRTMaker       // RelaX Srt File Maker (converter for sub files etc)
 ,uSRTCleaner     // RelaX SRT File Cleaner for B6
 ,IdIcmpClient    // Part of Indy Delphi Components Suite
 ;

{$R *.dfm}

{ TdmListener }

Type // object to synchronise display events with main thread
  TDisplayObject=Class(Tobject)
  Public
    dlstatus : Tdlstatus;
    Parent : TWinControl;
    Filename : String;
    Progress : Integer;
    MaxValue : Integer;
    MinValue : Integer;
    RemoteIP : String;
    Procedure Start;
    Procedure Update;
    Procedure Stop;
    Function ForceStop:Boolean;
  End;

{ TDisplayObject }
  procedure TDisplayObject.Stop;
  begin
    if not assigned(frmMain) then exit;
    FreeAndNil(dlstatus);
  end;

  procedure TDisplayObject.Start;
  begin
    if not assigned(frmMain) then exit;
    dlstatus := tdlstatus.Create(Nil);
    dlstatus.Align:= alTop;
    dlstatus.Parent := Parent;
    dlstatus.lblFilename.Caption := Filename;
    dlstatus.PbarPosition.MaxValue := MaxValue;
    dlstatus.PbarPosition.Progress := Progress;
    dlstatus.PbarPosition.MinValue := MinValue;
    dlstatus.lblRemoteIP.Caption := RemoteIP;
  end;

  function TDisplayObject.ForceStop: Boolean;
  begin
    Result := dlstatus.Stop;
  end;


  procedure TDisplayObject.Update;
  begin
    if not assigned(frmMain) then exit;
    dlstatus.lblFilename.Caption := Filename;
    dlstatus.PbarPosition.MaxValue := MaxValue;
    dlstatus.PbarPosition.Progress := Progress;
    dlstatus.PbarPosition.MinValue := MinValue;
    dlstatus.lblRemoteIP.Caption := RemoteIP;
  end;


Type // object to synchronise display events with main thread
  TLogUpdate = Class
  Public
    Line:String;
    Memo:TMemo;
    Procedure DoAddLog;
  End;
  Procedure TLogUpdate.DoAddLog;
  Begin
    LockWindowUpdate(memo.Handle);
    while Memo.Lines.Count >200 do
      Memo.lines.Delete(0);
    Memo.Lines.Add(Line);
    LockWindowUpdate(0);
  End;

// synchronized logger!
procedure TdmListener.Log(debuglevel: Integer; S: STring; AThread: TIdPeerThread;aIP:String = '000.000.000.000');
var
  Logger:TLogUpdate;
begin
  // no main form.. then nothing to log!
  if not assigned(frmMain) then
    exit;

  // Only log when allowed within debug level
  if StrToInt(frmMain.DebugLevel.text)>Debuglevel then
    exit;
  Try
    Logger := TLogUpdate.Create;
    Logger.Line := max(aIP,16)+Logger.Line+inttostr(debuglevel)+' '+s;
    Logger.Memo := frmMain.Memo1;
    if assigned(aThread) then
      aThread.Synchronize(Logger.DoAddLog)
    Else
      Logger.DoAddLog;
    FreeAndNil(Logger);
  except
    on e:exception do showmessage(e.message);
  End;
end;


  constructor tempconfigobject.create;
  begin
    subson := True;
    Framerate := 0;
    Shutdown := False;
  end;


function TdmListener.DoConfig(aUserIP, aConfigOption: String):TempConfigObject;
var
  mytempconfigobject:TempConfigObject;
  x:Longint;
begin

  if not assigned(ipConfigList) then
    ipConfigList := TStringlist.Create;
  x := ipConfigList.indexof(aUserIP);
  if ipConfigList.indexof(aUserIP)>-1 then
    mytempconfigobject := TempConfigObject(ipConfiglist.objects[x])
  else
    begin
      mytempconfigobject := tempconfigobject.create;
      ipconfiglist.AddObject(aUserIP,mytempconfigobject)
    end;


  case StrToInt(aConfigOption) of
    0  : ; // just get the config object!
    1  : mytempconfigobject.subson := True;// subs on
    2  : mytempconfigobject.subson := False;// subs off
    3  : mytempconfigobject.FrameRate := 0;// sub convert auto
    4  : mytempconfigobject.FrameRate := 23.976;// sub convert 23.976
    5  : mytempconfigobject.FrameRate := 23.980;// sub convert 23.980
    6  : mytempconfigobject.FrameRate := 24.000;// sub convert 24.000
    7  : mytempconfigobject.FrameRate := 25.000;// sub convert 25.000
    8  : mytempconfigobject.FrameRate := 29.970;// sub convert 29.970
    9  : mytempconfigobject.FrameRate := 30.000;// sub convert 30.000
    10 : mytempconfigobject.Shutdown := true;// Shutdown on
    11 : mytempconfigobject.Shutdown := false;// Shutdown off
  end;
  result := mytempconfigobject;
end;




Function GetMediaFilesFromPath(aXMLNode:TJanXMLNode2;Path:String):String;
var
  Diri:TSearchRec;
  res:Integer;
  Ext:String;
  RecursiveXML   : TJanXMLNode2;
  FlattenXML     : TJanXMLNode2;
  ExtentionsXML  : TJanXMLNode2;
  LoopXML        : TJanXMLNode2;
  Flatten:Boolean;
  doDirectories:Boolean;
  Exts:String;
  Tmp:STring;
  x:Integer;
  ExtsList:TStringlist;
Begin
// hier ophalen van de spullen in de xmlelement
  FlattenXML     := GetNodeByName('FLATTEN'    ,aXMLNode.nodes);
  if assigned(FlattenXML) then Flatten := FlattenXML.text = 'Y'
                          else Flatten := False;

  RecursiveXML   := GetNodeByName('RECURSIVE'  ,aXMLNode.nodes);
  if assigned(RecursiveXML) then doDirectories := RecursiveXML.text = 'Y'
                            else doDirectories := True;

  //

  exts := '';
  LoopXML := aXMLNode;
  ExtentionsXML  := GetNodeByName('EXTENTIONS' ,LoopXML.nodes);
  if assigned(extentionsXML) then
    exts := extentionsXML.text;
  while (exts = '') and assigned(LoopXML.parentNode) do
    begin
      LoopXML := LoopXML.parentNode;
      ExtentionsXML  := GetNodeByName('EXTENTIONS' ,LoopXML.nodes);
      if assigned(extentionsXML) then
        exts := extentionsXML.text;
    end;
  if exts = '' then
    exts := frmMain.Extentions.text;
  if exts = '' then
    exts := 'AVI;MP2;MP3;MP4;MPG;WMA;WMV;BMP;MPEG;JPEG;JPG;XBE;DAT;DIVX';
  Exts := Exts + ';';

  ExtsList := TStringList.Create;
  tmp:='';
  While length(exts)>0 do
    Begin
      case exts[1] of
        ';' : begin
                if tmp<>'' then
                  extsList.Add(uppercase(tmp));
                tmp:='';
              end;
         else begin
                tmp := tmp + exts[1];
              end;
      end;
      delete(exts,1,1);
    end;
  ExtsList.Sort;
  ExtsList.Sorted:=True;

  Result := '';
  if (length(path)>0) and (path[length(path)]<>'\') then
    path:=path+'\';
  Res := Findfirst(path+'*.*',faanyfile,diri);
  While Res=0 do
    Begin
      if diri.Attr and faDirectory = faDirectory then
        Begin
          if (diri.Name <>'.') and (diri.name<>'..') then
            begin
              if Flatten then
                Result := Result + GetMediaFilesFromPath(aXMLNode,Path+diri.Name)
              else
                if doDirectories then
                  Result := Result + '<ITEM><ATTRIB>16</ATTRIB><PATH>'+path+diri.name+'</PATH></ITEM>'+#13#10;
            end;
        End
      Else
        Begin
          Ext := Uppercase(ExtractFileExt(Diri.name));
          if (length(ext)>0) and (Ext[1]='.') then
            Begin
              delete(ext,1,1);
            End;
          if ExtsList.find(ext,x)
{           (Ext = 'AVI')
            or (Ext = 'MP2')
            or (Ext = 'MP3')
            or (Ext = 'MP4')
            or (Ext = 'MPG')
            or (Ext = 'WMA')
            or (Ext = 'WMV')
            or (Ext = 'BMP')
            or (Ext = 'MPEG')
            or (Ext = 'JPEG')
            or (Ext = 'JPG')
            or (Ext = 'XBE')
            or (Ext = 'DAT')
            or (Ext = 'DIVX')}
          then
            Begin
              Result := Result + '<ITEM><ATTRIB>128</ATTRIB><PATH>'+path+diri.name+'</PATH></ITEM>'+#13#10;
            End;
        End;
      Res := Findnext(diri);
    End;
  FindClose(diri);
  FreeAndNil(ExtsList);
End;

function GetItemListFromXMLNode(XMLNode:TjanXMLNode2;UserIP:String):String;
var
  x: Longint;

  attribute : TJanXMLNode2;
  Path,item : TJanXMLNode2;

  IpList    : TJanXMLNode2;
  Hidden    : TJanXMLNode2;
  DisplayName : TJanXMLNode2;
  Intergrated : TJanXMLNode2;

  Skip : Boolean;
  AllowedIPList : TStringlist;
  AllowedIPIdx : Longint;

  isIntergrated : Boolean;

  DispName : String;
Begin
  Result := '';


  For x:=0 to XMLNode.nodes.Count-1 do
    Begin
      item :=TjanXMLNode2(XMLNode.nodes.Items[x]);
      if uppercase(item.name)='ITEM' then
        Begin


          Attribute   := GetNodeByName('ATTRIBUTE'  ,item.nodes);
          Path        := GetNodeByName('PATH'       ,item.nodes);

          IpList      := GetNodeByName('ALLOWEDIP'  ,item.nodes); // done;
          Hidden      := GetNodeByName('HIDDEN'     ,item.nodes); // done;

          DisplayName := GetNodeByName('DISPLAYNAME',item.nodes);
          Intergrated := GetNodeByName('INTERGRATE' ,item.nodes);


          Skip := False;

          if assigned(Hidden) and (Hidden.text='Y') then Skip := True;

          if assigned(IpList) and (IpList.text <> '') and (skip=false)then
            Begin
              AllowedIPList := TStringlist.Create;
              AllowedIPList.Text := ipList.text;
              // skip current tag if not allowed!
              Skip := Not AllowedIPList.Find(UserIP,AllowedIPIdx);
              FreeAndNil(AllowedIPList);
            End;

          if assigned(Intergrated) then isIntergrated := Intergrated.text='Y'
                                   else isIntergrated := False;

          DispName := Path.text;
          if assigned(DisplayName) and (displayname.text<>'') then
            DispName := DispName+VirtualDisplayTag+DisplayName.text;
          if not skip then
            case StrToInt(attribute.text) of
              128 : Begin // file
                      Result := Result + '<ITEM><ATTRIB>128</ATTRIB><PATH>'+path.text+'</PATH></ITEM>'+#13#10;
                    end;
              16 : Begin // directory
                      if isIntergrated then // Flatten
                        Result := Result + GetMediaFilesFromPath(item,path.text)
                      Else                   // Show As Directory for B6
                        Result := Result + '<ITEM><ATTRIB>16</ATTRIB><PATH>'+DispName+'</PATH></ITEM>'+#13#10;
                   End;
              0 : Begin // virtual path
                    DispName := DispName+VirtualDisplayTag+DispName;
                    Result := Result + '<ITEM><ATTRIB>16</ATTRIB><PATH>'+DispName+'</PATH></ITEM>'+#13#10;
                  End;
              1 : Begin // Configuration tag
                    DispName := DispName+VirtualDisplayTag+DispName;
                    Result := Result + '<ITEM><ATTRIB>16</ATTRIB><PATH>'+DispName+'</PATH></ITEM>'+#13#10;
                  End;
            end;
        End;
    End;
End;

Function HostStillThere(aIP:String):Boolean;
var
  tmpIcmp:TIdIcmpClient;
Begin
  Result := True;
  tmpIcmp:=TIdIcmpClient.Create(Nil);
  tmpIcmp.ReceiveTimeout := 300;
  Try
    tmpIcmp.Host := aIP;
    tmpIcmp.Ping();
    if tmpIcmp.ReplyStatus.ReplyStatusType = rsTimeOut then
      Result := False;
  Except
    on Exception do
      Result := False;
  End;
  FreeAndNil(tmpIcmp);
End;

Function GetDivRatio(x:Int64):Integer;
Begin
  Result := 1;
  While x / Result > 2147483640 do
    Begin
      Result := Result + 1;
    End;
End;
// this function is called by indy tcpserver
// this is the handler from the xbox mediaplayer requests
procedure TdmListener.IdTCPServer1Connect(AThread: TIdPeerThread);
var
  Greeting : String;
  command : String;
//  tmp:String;
  param : String;
  x:Longint;
  done : Boolean;
  MediaStream : TStream;
  Response : String;
  ResponseData : String;
  buf:array[1..800000] of byte;
  bufzise : Longint;
  tmpAviInfo:Avi_info;
  DisplayObject : TDisplayObject;

  Title: String;
  Artists : String;
  Summary : String;
  Picture : String;

  Timedout : Boolean;
  Mp3Info:TMp3Object;
  UserIP : String;
  tmpsl : TStringlist;

  tmpxml : TJanXMLNode2;
  attributeXML : TJanXMLNode2;
  pathXML : TJanXMLNode2;
  itemXML : TJanXMLNode2;
  ConfigurationtagXML : TJanXMLNode2;

  xbmpVersion : String;
  sl:TStringlist;
  Divratio:Longint;
  Foundone : Boolean;

  SubtitleStream:Boolean;
begin
  // need nodelay for FAST streaming! Thanks runtime!
  // offcause also tanks to runtime for supplying the .NET source of the
  // xstream server.. this made it alot easyer to know what requests are
  // to be expected!

  aThread.Synchronize(frmMain.addConnection);
  aThread.Priority := tpHighest;
  DisplayObject := Nil;
  timedout := False;
  done := false;
  DivRatio := 1;
  Try
    (aThread.Connection.IOHandler as TIdIOHandlerSocket).Binding.SetSockOpt(Id_IPPROTO_TCP, Id_TCP_NODELAY, PChar(@Id_SO_True), SizeOf(Id_SO_True));
    UserIP := (AThread.Connection.IOHandler as TIdIOHandlerSocket).Binding.PeerIP;
    Log(0,'---------------------------------------------------------------------------',aThread,UserIP);
    log(2,'Incomming connection from '+(AThread.Connection.IOHandler as TIdIOHandlerSocket).Binding.PeerIP,aThread,UserIP);
    Log(1,'Waiting for greeting',aThread,UserIP);
    Greeting := aThread.Connection.ReadLn(#10,4000,2000);
    xbmpVersion := 'B6';// or higher
    if Greeting = 'HELLO XSTREAM 5.0' then
      xbmpVersion := 'B5';
    if Greeting = 'HELLO XSTREAM 6.0' then
      xbmpVersion := 'B6';
    Log(1,'received greeting :'+greeting,aThread,UserIP);
    Log(1,'Sending greeting : HELLO XBOX!',aThread,UserIP);
    aThread.connection.Write('HELLO XBOX!');

    AThread.Connection.ReadTimeout:=2000; // 5 second timeout!
    Log(1,'Waiting in endles loop for commands',aThread,UserIP);
  except
    on exception do
      done := True;
  End;
  While not done do
    Try
      done := False;
      Response := '-1 UNKNOWN ERROR';
      ResponseData := '';
      bufzise := 0;
      Try
        command := aThread.Connection.ReadString(4);
      except
        on exception do
          command := '';
      End;
      if aThread.Connection.Connected=false then
        done := True;
      if command='' then
        Begin
          if not timedout then
            log(4,'command timeout',aThread,UserIP);
          x:=downloadiplist.IndexOf(UserIP);
          // stop Mediastream when ip is downloading other media!
          // should stop the hanging files bug

          if (x >-1) and (DownLoadIpList.Objects[x]<>MediaStream) then
            FreeAndNil(MediaStream);

          if assigned(mediaStream) and (MediaStream.Position = MediaStream.Size) then
            FreeAndNil(MediaStream); // free when end!
          // only wait for timeouts when streaming media;

          // ping the xbox .. ! on pingtimeout disconnect
          if Assigned(mediaStream) and (not HostStillThere(UserIP)) then
            FreeAndNil(MediaStream); // free when end!

          done := not assigned(MediaStream);
          timedout := true;
        end
      else
        Begin
          if timedout then
            Log(4,'End of timeout',aThread,UserIP);
          timedout := false;
          param := aThread.Connection.ReadString(aThread.Connection.Inputbuffer.Size);

          if command<>'READ' then
            Log(1,'Command gotten : '+command,aThread,UserIP);

          if command = '*CAT' then
            Begin
              x:=IpCurrentPath.IndexOf(UserIP);
              if x=-1 then
                Begin // ip is not yet in List! adding root!
                  tmpsl := TStringlist.Create;
                  tmpsl.AddObject('',shares);
                  IpCurrentPath.AddObject(UserIP,tmpsl);
                End
              Else
                Begin // ip is already downloading listing;
                  tmpsl := TStringlist(IpCurrentPath.Objects[x]);
                End;
              // tmpsl should be filled with the backlog of the current user (ip)
              if param='' then
                begin // resetting the directory to root when no param is given (only at startup! and B5)
                  while tmpsl.count>1 do tmpsl.delete(tmpsl.count-1);
                end;

              Log(1,'param : '+param,aThread,UserIP);
              if Length(param)>0 then
                Begin

                  if GetParam(1,param)='BACK' then
                    Begin // going back a directory
                      if tmpsl.Count >1 then // only back when we are not in the root!
                        tmpsl.Delete(tmpsl.count-1); // we just want to go back one :)
                    End
                  Else
                    Begin// going forward a directory
                      tmpXML := TJanXMLNode2(tmpsl.Objects[tmpsl.count-1]);
                      if assigned(tmpXML) then
                        Begin
                          foundone := False;
                          for x:=0 to tmpXML.nodes.Count-1 do
                            Begin // Going trough the nodelist seeking the parent
                              itemXML := TJanXMLNode2(tmpXML.nodes.Items[x]);
                              if itemXML.name = 'ITEM' then
                                Begin
                                  pathXML := GetNodeByName('PATH',itemXML.nodes);
                                  if assigned(pathXML) and (pathXML.text=GetParam(1,param)) then
                                    Begin
                                      tmpSL.AddObject(pathXML.text,itemXML);
                                      foundone := True;
                                    End;
                                End; // if itemXML.name = 'ITEM'
                            End; // for x:=0 to tmpXML.nodes.count-1
                          if not foundone then
                            tmpsl.Add(GetParam(1,param))
                        End // if assigned(tmpxml)
                      Else
                        Begin // no xml object so must be in directory
                          tmpsl.Add(GetParam(1,param))
                        End;
                    End;
                End;

              ResponseData := '<catalogue>'+#13#10;



              attributeXML := Nil;
              ConfigurationtagXML := Nil;

              if tmpsl.Objects[tmpsl.Count-1]<>nil then
                ConfigurationtagXML := GetNodeByName('CONFIGURATIONTAG',TJanXMLNode2(tmpsl.Objects[tmpsl.Count-1]).Nodes);

              if assigned(ConfigurationtagXML) then
                begin
                  Log(0,'Configuration option '+ConfigurationtagXML.text,AThread,UserIP);
                  doConfig(userIP,ConfigurationtagXML.text);
                  tmpsl.Delete(tmpsl.Count-1);
                end;

              if tmpsl.Objects[tmpsl.Count-1]<>nil then
                attributeXML        := GetNodeByName('ATTRIBUTE'       ,TJanXMLNode2(tmpsl.Objects[tmpsl.Count-1]).Nodes);

              if   (    (Assigned(attributeXML))
                     and (  (attributeXML.text = '0') // virtual directory
                          or(attributeXML.text = '1') // configuration tag
                         )
                   )
                or (tmpsl.Objects[tmpsl.Count-1] = shares)

              then
                // show filelist from a virtual directory!
                ResponseData := ResponseData+GetItemListFromXMLNode(TJanXMLNode2(tmpsl.objects[tmpsl.count-1]),UserIP)
              else
                begin
                  // showfileslist from real directory
                  itemXML := Nil;
                  // Searching for the last configured object!
                  for x:=0 to tmpsl.Count-1 do
                    if tmpsl.Objects[x]<>nil then
                      itemXML := TJanXMLNode2(tmpsl.Objects[x]);
                  if itemxml<>nil then
                    ResponseData := ResponseData+GetMediaFilesFromPath(ItemXML,tmpsl.Strings[tmpsl.count-1]) // flatten
                end;

              ResponseData := ResponseData + '</catalogue>'+#13#10;
              Response := inttostr(length(ResponseData))+' OK';
              Done := True;
            End;
          if command = 'OPEN' then
            Begin
              log(1,'Opening file :'+GetParam(1,param),athread,userip);
              SubTitleStream:=Uppercase(ExtractFileExt(GetParam(1,param)))='.SRT';
              if subtitlestream and (DoConfig(UserIP,'0').subson=false) then
                begin
                  log(1,'Subtitle is off in configuration.. reporting no srt found!',aThread,userip);
                  subtitlestream := False;
                  param := '';
                end;

              if fileExists(GetParam(1,param)) then
                Begin
                  MediaStream := TFileStream.Create(GetParam(1,param),fmOpenRead+fmShareDenyNone);
                  Log(1,' File Found ',aThread,UserIP);
                end
              else
                Begin
                  if SubTitleStream then
                    Begin
                      Log(1,'Trying to save the day by searching for other subtitle files and convert them',aThread,UserIP);
                      sl:=TStringlist.Create;
                      sl.Text := uSRTMaker.GetSRTList(GetParam(1,param),frmMain.SubsDirectory.text,DoConfig(UserIP,'0').Framerate);
                      if sl.Text<>'' then
                        Begin
                          Log(1,' Converting done.. Serving the stream!',aThread,UserIP);
                          MediaStream := TMemoryStream.Create;
                          sl.SaveToStream(MediaStream);
                        End
                      Else
                        Log(1,' I Could not find any file to convert',aThread,UserIP);
                      FreeAndNil(sl);
                    End
                End;

              if assigned(MediaStream) then
                Begin
                  if SubtitleStream then
                    Begin
                      Log(1,'Subtitle SRT stream cleaning for B6',athread,UserIP);
                      MediaStream := usrtCleaner.GetCleanSRTStream(MediaStream,frmMain.SubsCleaningStripes.text='Y',frmMain.SubsCleaningReduce.text='Y',StrToInt(frmMain.SubsCleaningMax.text));
                    End;

                  DisplayObject := TDisplayObject.Create;
                  if assigned(frmMain) then
                    DisplayObject.Parent := frmMain.Scrollbox1;
                  DisplayObject.Filename := extractfilename(GetParam(1,param));
                  DisplayObject.Progress := 0;
                  DivRatio := GetDivRatio(MediaStream.Size);
                  Log(0,'DivRatio = '+inttostr(divRatio),aThread,userIP);
                  DisplayObject.MaxValue := MediaStream.Size div divratio;
                  DisplayObject.MinValue := 0;
                  DisplayObject.RemoteIP := UserIP;
                  // keeping track of downloads per ip
                  // to disconnect every old download from same ip
                  x:=downloadiplist.IndexOf(UserIP);
                  if x=-1 then // ip is not yet downloading;
                    DownloadipList.AddObject(UserIP,MediaStream)
                  Else // ip is already downloading
                    DownloadipList.Objects[x] := MediaStream;
                  AThread.Synchronize(DisplayObject.Start);

                  MediaStream.Position := 0;
                  Response := Inttostr(MediaStream.Size)+' OK';
                  Done := False;
                End
              else
                Begin
                  Response := '-1 UNABLE TO OPEN';
                  Log(1,' File not found',aThread,UserIP);
                  Done:=True;
                End;
            End;

          if command = 'INFO' then
            Begin
              Log(1,'reading INFO for :'+GetParam(1,param),aThread,UserIP);
              ResponseData := '<media><share>notimportant</share>';
              Picture := '';
              Title := 'Me';
              Artists := 'Duno';
              Summary := 'Something else';
              if Uppercase(ExtractFileExt(getParam(1,param)))='.AVI' then
                Begin
                  tmpAviInfo := GetAviInfo(getParam(1,param));
                  Title   := 'Compressor  '+tmpAviInfo.Compressor;
                  Artists := 'Aspect    '+tmpAviInfo.Aspect;
                  summary := 'Duration  '+tmpAviInfo.Duration+#13#10+
                             'Framerate '+tmpAviInfo.FrameRate;
                End;
              if Uppercase(ExtractFileExt(GetParam(1,param)))='.MP3' then
                Begin
                  Mp3Info:=TMp3Object.Create(Nil);
                  Mp3Info.Mp3File := getParam(1,param);
                  Title   := 'Bitrate   '+Mp3Info.MpegType+'  '+Mp3Info.Layer+'  '+Mp3Info.BitRate+'kbps  '+Mp3Info.ChannelMode;
                  Artists := 'Size        '+Mp3Info.Size;
                  Summary := 'Duration  '+Mp3Info.Time;
                  FreeAndNil(Mp3Info);
                End;
              ResponseData := ResponseData+'<title>'+Title+'</title>';
              ResponseData := ResponseData+'<artists>'+Artists+'</artists>';
              ResponseData := ResponseData+'<summary>'+summary+'</summary>';
              ResponseData := ResponseData+'</media>';
              if (title = 'Me') and (Artists='Duno') and (Summary='Something else') then
                begin
                  Responsedata := '';
                  Response := '-1 ERROR';
                end
              else
                Response := inttostr(length(ResponseData))+' OK';
              Done := True;
            end;

          if command = 'READ' then
            Begin
              Done := False;
              if assigned(MediaStream) then
                Begin
                  MediaStream.Position := StrToInt64(GetParam(1,param)); // integer64
                  bufzise := MediaStream.Read(buf,StrToInt(GetParam(2,param)));
                  if assigned(displayobject) then
                    Begin
                      DisplayObject.Progress := MediaStream.position div divratio;
                      AThread.Synchronize(DisplayObject.Update);
                    End;
                  Response := inttostr(bufzise)+' OK';
                  ResponseDAta := '';
                End
              Else
                Begin
                  Response := '-1 READ FAILED';
                  Done := True;
                End;
            end;
          if command = 'TELL' then
            Begin
              if assigned(MediaStream) then
                Response := Inttostr(MediaStream.Position)+' OK'
              Else
                Response := '-1 TELL FAILED';
            end;

          if command = 'CLSE' then
            Begin
              FreeAndNil(MediaStream);
              Response := '0 OK';
              Done := True;
            end;

          if command = 'DBUG' then
            Begin
              Response := '0 OK';

            end;
          Response := MAX(Response,32);
          aThread.Connection.WriteBuffer(Response[1],Length(Response),true);
          if ResponseData <> '' then
            aThread.Connection.WriteBuffer(Responsedata[1],Length(Responsedata),true);
          if bufzise <> 0 then
            aThread.Connection.WriteBuffer(buf,bufzise,true);
          if assigned(DisplayObject) and DisplayObject.ForceStop then
            Done:=True;

          if done then
            aThread.Connection.Disconnect
          else
            application.ProcessMessages;

        End;
    except
      on e:exception do
        Begin
          FreeAndNil(MediaStream);
          log(5,e.Message,aThread,UserIP);
          Done := True;
          aThread.Connection.Disconnect;
        End;
    End;
  Log(0,'End of it all',aThread,UserIP);
  if assigned(DisplayObject) then
    Try
      AThread.Synchronize( DisplayObject.Stop );
      DisplayObject.Free;
    except
      on e:Exception do
        log(5,e.Message,aThread,UserIP);
    End;
  aThread.Synchronize(frmMain.DeleteConnection);
//  Application.ProcessMessages;
end;

procedure TdmListener.IdTCPServer1Disconnect(AThread: TIdPeerThread);
begin
  Log(0,'Disconnected',athread,'connection lost');
end;

procedure TdmListener.DataModuleCreate(Sender: TObject);
begin
  downloadiplist := TStringlist.Create;
  IpCurrentPath := TStringList.Create;
  IdTCPServer1.DefaultPort := StrToInt(frmMain.ListenPort.text);
  IdTCPServer1.Active := True;
  log(0,'Listener started',nil);
end;

procedure TdmListener.DataModuleDestroy(Sender: TObject);
begin
  freeandnil(downloadiplist);
  FreeAndNil(IpCurrentPath);
  IdTCPServer1.Active := False;
  log(0,'Listener stopped',nil);
end;


{ tempconfigobject }


end.
